//import 'dart:core';
//import 'package:meta/meta.dart';
import 'package:flutter/material.dart' as f;
import 'package:particle_ui/particle.dart';

class Surface extends StatefulWidget {
  final Size size;
  final Shape shape;
  final Fill fill;
  final Opacity opacity;
  final Elevation elevation;
  final EdgeInsets margin;
  final EdgeInsets padding;
  final Alignment alignment;

  final List<MotionType> motions;

  final f.VoidCallback onTap;

  final Widget child;

  const Surface({Key key, this.size, this.shape = Shapes.rectangular, this.fill = const Color('FFFFFF', opacity: Opacities.transparent), this.opacity, this.elevation = const Elevation(0), this.margin, this.padding, this.alignment = Alignments.topLeft, this.motions, this.onTap, this.child}) : super(key: key);

  @override
  _SurfaceState createState() => _SurfaceState();
}

class _SurfaceState extends State<Surface> with f.SingleTickerProviderStateMixin {
  f.AnimationController _controller;
  f.Animation<double> _opacityAnimation;

  @override
  void initState() {
    super.initState();

    if (widget.motions != null) {
      _controller = f.AnimationController(vsync: this, duration: Duration(milliseconds: 300));

      widget.motions.forEach((motion) {
        switch (motion) {
          case MotionType.fadeIn: {
            if (widget.fill != null) {
              _opacityAnimation = f.Tween<double>(
                begin: .0,
                end: widget.opacity != null ? widget.opacity.value : 1.0
              ).animate(_controller)
              /*..addStatusListener((state) {
                if (state == f.AnimationStatus.completed) {

                } else {

                }
              })*/
              ..addListener(() {
                setState(() {});
              });
            }
          }
          break;

          case MotionType.fadeOut: {

          }
          break;
        }
      });

      _controller.forward();
    }
  }

  @override
  void dispose() {
    super.dispose();

    if (widget.motions != null) {
      _controller.dispose();
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget _widget = widget.child;

    if (widget.onTap != null) {
      _widget = f.Material(
        color: f.Color(0x00000000),
        child: f.InkWell(
          onTap: widget.onTap,
          child: _widget
        )
      );
      /*_widget = f.InkWell(
        onTap: widget.onTap,
        child: _widget
      );*/
    }

    f.Color _color;
    if (widget.fill is Color) {
      Color _colorBackground = widget.fill;
      _color = _colorBackground.to32BitARGBColor();
    } else if (widget.fill is LinearGradient) {
      _color = f.Color(0x00000000);

      LinearGradient _gradient = widget.fill;
      List<f.Color> _colors = List<f.Color>();
      _gradient.colors.forEach((color) {
        _colors.add(color.to32BitARGBColor());
      });

      f.Color _blendColor = f.Color.alphaBlend(f.Color.fromRGBO(f.Color(0xff000000).red, f.Color(0xff000000).green, f.Color(0xff000000).blue, 1), f.Color.fromRGBO(_colors[0].red, _colors[0].green, _colors[0].blue, 1));
      f.Color _shadowColor = f.Color.fromRGBO(_blendColor.red, _blendColor.green, _blendColor.blue, _colors[0].opacity);

      _widget = f.Container(
        decoration: f.BoxDecoration(
          gradient: f.LinearGradient(begin: _gradient.begin, end: _gradient.end, colors: _colors),
          boxShadow: [f.BoxShadow(color: _shadowColor, blurRadius: widget.elevation.height)]
        ),
        child: _widget
      );
    }/* else if (widget.fill is ImageData) {

    }*/

    if (widget.padding != null) {
      _widget = f.Padding(padding: f.EdgeInsets.fromLTRB(widget.padding.left, widget.padding.top, widget.padding.right, widget.padding.bottom), child: _widget);
    }

    double _minWidth = .0;
    double _minHeight = .0;
    double _maxWidth = double.infinity;
    double _maxHeight = double.infinity;
    double _width = double.infinity;
    double _height = double.infinity;
    bool isConstraints = false;
    if (widget.size != null) {
      if (widget.size.minWidth != null || widget.size.minHeight != null || widget.size.maxWidth != null || widget.size.maxHeight != null) {
        isConstraints = true;

        _minWidth = widget.size.minWidth ?? widget.size.width ?? _minWidth;
        _minHeight = widget.size.minHeight ?? widget.size.height ?? _minHeight;
        _maxWidth = widget.size.maxWidth ?? widget.size.width ?? _maxWidth;
        _maxHeight = widget.size.maxHeight ?? widget.size.height ?? _maxHeight;
        _width = _minWidth ?? _maxWidth; //widget.size.width ?? widget.size.constraints.minWidth ?? widget.size.constraints.maxWidth ?? _width;
        _height = _minHeight ?? _maxHeight; //widget.size.height ?? widget.size.constraints.minHeight ??  widget.size.constraints.maxHeight ?? _height;

        _widget = f.ConstrainedBox(
          constraints: f.BoxConstraints(
            minWidth: _minWidth,
            minHeight: _minHeight,
            maxWidth: _maxWidth,
            maxHeight: _maxHeight
          ),
          child: _widget
        );
      } else {
        _width = widget.size.width ?? _width;
        _height = widget.size.height ?? _height;

        _widget = SizedBox(width: _width, height: _height, child: _widget);
      }
    }

    f.ShapeBorder _shapeBorder;
    f.BorderRadius _borderRadius;
    if (widget.shape is Rectangular) {
      Rectangular _shape = widget.shape;
      if (_shape.corner == null) {
        _shapeBorder = f.ContinuousRectangleBorder();
      } else {
        if (_shape.corner is RoundedCorner) {
          RoundedCorner _corner = _shape.corner;
          _borderRadius = f.BorderRadius.only(topLeft: f.Radius.circular(_corner.topLeft ?? 0), topRight: f.Radius.circular(_corner.topRight ?? 0), bottomLeft: f.Radius.circular(_corner.bottomLeft ?? 0), bottomRight: f.Radius.circular(_corner.bottomRight ?? 0)); //f.BorderRadius.circular(_corner.radius);
        } else {

        }
        _shapeBorder = f.RoundedRectangleBorder(borderRadius: _borderRadius);
        _widget = f.ClipRRect(
          borderRadius: _borderRadius,
          child: _widget
        );
      }
    } else if (widget.shape is Circular) {
      if (_width != null && _height != null) {
        if (_width > _height) {
          _borderRadius = f.BorderRadius.circular(_height/2);
        } else {
          _borderRadius = f.BorderRadius.circular(_width/2);
        }
        _shapeBorder = f.RoundedRectangleBorder(borderRadius: _borderRadius);
        _widget = f.ClipRRect(
          borderRadius: _borderRadius,
          child: _widget
        );
      } else {
        _shapeBorder = f.ContinuousRectangleBorder();
      }
    } else {
      // TODO: Support more shapes
    }

    _widget = _BaseSurface(
      shape: _shapeBorder,
      color: _color,
      elevation: widget.elevation.height,
      alignment: widget.alignment,
      child: _widget
    );

    double _opacity;
    if (widget.opacity != null) {
      _opacity = widget.opacity.value;
    }
    if (_opacityAnimation != null) {
      _opacity = _opacityAnimation.value;
    }

    if (_opacity != null) {
      _widget = f.Opacity(
        opacity: _opacity,
        child: _widget
      );
    }

    if (widget.size != null && isConstraints == false) {
      _widget = SizedBox(width: _width, height: _height, child: _widget);
    }

    if (widget.margin != null) {
      _widget = f.Padding(padding: f.EdgeInsets.fromLTRB(widget.margin.left, widget.margin.top, widget.margin.right, widget.margin.bottom), child: _widget);
    }

    return _widget;
  }
}


class _BaseSurface extends f.ImplicitlyAnimatedWidget {
  final f.ShapeBorder shape;
  final f.Clip clipBehavior;
  final f.Color color;
  final f.Color shadowColor;
  final double elevation;
  final Alignment alignment;
  final Duration duration;
  final f.Curve curve;
  final Widget child;

  const _BaseSurface({Key key, @required this.shape, this.clipBehavior = f.Clip.none, @required this.color, this.shadowColor = const f.Color(0xff000000), this.elevation, this.alignment, this.duration = const Duration(milliseconds: 200), this.curve = f.Curves.linear, @required this.child}) : assert(shape != null), assert(clipBehavior != null), assert(color != null), assert(shadowColor != null), assert(child != null), super(key: key, duration: duration, curve: curve);

  @override
  _BaseSurfaceState createState() => _BaseSurfaceState();
}

class _BaseSurfaceState extends f.AnimatedWidgetBaseState<_BaseSurface> {
  f.ShapeBorderTween _shapeTween;
  f.ColorTween _colorTween;
  f.ColorTween _shadowColorTween;
  f.Tween<double> _elevationTween;
  f.AlignmentTween _alignmentTween;

  @override
  void forEachTween(f.TweenVisitor<dynamic> visitor) {
    _shapeTween = visitor(_shapeTween, widget.shape, (dynamic value) => f.ShapeBorderTween(begin: value));
    _colorTween = visitor(_colorTween, widget.color, (dynamic value) => f.ColorTween(begin: value));
    _shadowColorTween = visitor(_shadowColorTween, _blendShadowColor(), (dynamic value) => f.ColorTween(begin: value));
    _elevationTween = widget.elevation != null ? visitor(_elevationTween, widget.elevation, (dynamic value) => f.Tween<double>(begin: value)) : null;
    _alignmentTween = widget.alignment != null ? visitor(_alignmentTween, widget.alignment, (dynamic value) => f.AlignmentTween(begin: value)) : null;
  }

  @override
  Widget build(BuildContext context) {
    Widget _widget = f.PhysicalShape(
      color: _colorTween.evaluate(animation),
      shadowColor: _shadowColorTween.evaluate(animation),
      clipper: f.ShapeBorderClipper(
        shape: _shapeTween.evaluate(animation),
        textDirection: f.Directionality.of(context),
      ),
      elevation: _elevationTween != null ? _elevationTween.evaluate(animation) : .0,
      child: widget.child
    );

    if (widget.alignment != null) {
      _widget = f.Align(
        alignment: _alignmentTween.evaluate(animation), child: _widget
      );
    }

    return _widget;
  }

  f.Color _blendShadowColor() { // If surface color is translucent, reduce shadow intensity and blend shadow color with the surface color
    if (widget.shadowColor.opacity > widget.color.opacity) {
      f.Color _blendColor = f.Color.alphaBlend(f.Color.fromRGBO(widget.shadowColor.red, widget.shadowColor.green, widget.shadowColor.blue, 1), f.Color.fromRGBO(widget.color.red, widget.color.green, widget.color.blue, 1));

      return f.Color.fromRGBO(_blendColor.red, _blendColor.green, _blendColor.blue, widget.color.opacity);
    } else {
      return widget.shadowColor;
    }
  }
}