import 'package:flutter/widgets.dart';
import 'package:particle_ui/particle.dart';

class Theme extends StatelessWidget {
  final ThemeSettings settings;
  final Widget child;

  const Theme({Key key, @required this.settings, @required this.child})
      : assert(settings != null),
        assert(child != null),
        super(key: key);

  static final ThemeSettings _fallbackTheme = ThemeSettings.fallback();

  static ThemeSettings of(BuildContext context) {
    final _InheritedTheme inheritedTheme = context.dependOnInheritedWidgetOfExactType<_InheritedTheme>();

    final ThemeSettings theme = inheritedTheme?.theme?.settings ?? _fallbackTheme;

    return theme;
  }

  @override
  Widget build(BuildContext context) {
    return _InheritedTheme(theme: this, child: child);
  }
}

class _InheritedTheme extends InheritedWidget {
  final Theme theme;

  const _InheritedTheme({Key key, @required this.theme, @required Widget child}) : assert(theme != null), super(key: key, child: child);

  @override
  bool updateShouldNotify(_InheritedTheme oldWidget) => theme.settings != oldWidget.theme.settings;
}